@@ -31,8 +31,10 @@
 
 template <class T> class QList;
 class QCommandLineParser;
+class QJsonObject;
 class QVariant;
 class KAboutData;
+class KPluginMetaData;
 namespace KCrash
 {
 Q_DECL_IMPORT void defaultCrashHandler(int sig);
@@ -134,6 +136,23 @@ public:
      */
     QString ocsUsername() const;
 
+    /**
+     * Creates a @c KAboutPerson from a JSON object with the following structure:
+     *
+     * Key        | Accessor
+     * -----------| ----------------------------
+     * Name       | name()
+     * Email      | emailAddress()
+     * Task       | task()
+     * Website    | webAddress()
+     * UserName   | ocsUsername()
+     *
+     * The @c Name and @c Task key are translatable (by using e.g. a "Task[de_DE]" key)
+     *
+     * @since 5.18
+     */
+    static KAboutPerson fromJSON(const QJsonObject &obj);
+
 private:
     /**
      * @internal Used by KAboutData to construct translator data.
@@ -174,7 +193,8 @@ public:
         QPL = 5,
         QPL_V1_0 = 5,
         GPL_V3 = 6,
-        LGPL_V3 = 7
+        LGPL_V3 = 7,
+        LGPL_V2_1 = 8 ///< @since 5.25
     };
 
     /**
@@ -269,7 +289,7 @@ private:
 
 /**
  * This class is used to store information about a program or plugin.
- * It can store such values as version number, program name, home page, email address
+ * It can store such values as version number, program name, home page, address
  * for bug reporting, multiple authors and contributors
  * (using KAboutPerson), license and copyright information.
  *
@@ -284,6 +304,42 @@ private:
  * applies to the old setCatalogName() and catalogName() members. But see also
  * K4AboutData in kde4support as a compatibility class.
  *
+ * Example:
+ * Setting the metadata of an application using KAboutData in code also relying
+ * on the KDE Framework modules KI18n and KDBusAddons:
+ * @code
+ * // create QApplication instance
+ * QApplication app(argc, argv);
+ * // setup translation string domain for the i18n calls
+ * KLocalizedString::setApplicationDomain("foo");
+ * // create a KAboutData object to use for setting the application metadata
+ * KAboutData aboutData("foo", i18n("Foo"), "0.1",
+ *                      i18n("To Foo or not To Foo"),
+ *                      KAboutLicense::LGPL,
+ *                      i18n("Copyright 2017 Bar Foundation"), QString(),
+ *                      "https://www.foo-the-app.net");
+ * // overwrite default-generated values of organizationDomain & desktopFileName
+ * aboutData.setOrganizationDomain("barfoundation.org");
+ * aboutData.setDesktopFileName("org.barfoundation.foo");
+ *
+ * // set the application metadata
+ * KAboutData::setApplicationData(aboutData);
+ * // in GUI apps set the window icon manually, not covered by KAboutData
+ * // needed for environments where the icon name is not extracted from
+ * // the information in the application's desktop file
+ * QApplication::setWindowIcon(QIcon::fromTheme(QStringLiteral("foo")));
+ * // with the application metadata set, register to the D-Bus session
+ * KDBusService programDBusService(KDBusService::Multiple | KDBusService::NoExitOnFailure);
+ *
+ * // integrate with commandline argument handling
+ * QCommandLineParser parser;
+ * aboutData.setupCommandLine(&parser);
+ * // setup of app specific commandline args
+ * [...]
+ * parser.process(app);
+ * aboutData.processCommandLine(&parser);
+ * @endcode
+ *
  * @short Holds information needed by the "About" box and other
  * classes.
  * @author Espen Sand (espen@kde.org), David Faure (faure@kde.org)
@@ -295,8 +351,15 @@ public:
 
     /**
      * Returns the KAboutData for the application.
-     * This contains information such as authors, license, etc.
-     * provided that the main() called setApplicationData.
+     *
+     * This contains information such as authors, license, etc.,
+     * provided that setApplicationData has been called before.
+     * If not called before, the returned KAboutData will be initialized from the
+     * equivalent properties of QCoreApplication (and its subclasses),
+     * if an instance of that already exists.
+     * For the list of such properties see setApplicationData
+     * (before 5.22: limited to QCoreApplication::applicationName).
+     * @see setApplicationData
      */
     static KAboutData applicationData();
 
@@ -304,8 +367,17 @@ public:
      * Sets the application data for this application.
      *
      * In addition to changing the result of applicationData(), this initializes
-     * the relevant properties of QCoreApplication (and its subclasses) with
-     * information from @p aboutData.
+     * the equivalent properties of QCoreApplication (and its subclasses) with
+     * information from @p aboutData, if an instance of that already exists.
+     * Those properties are:
+       <ul>
+       <li>QCoreApplication::applicationName</li>
+       <li>QCoreApplication::applicationVersion</li>
+       <li>QCoreApplication::organizationDomain</li>
+       <li>QGuiApplication::applicationDisplayName</li>
+       <li>QGuiApplication::desktopFileName (since 5.16)</li>
+       </ul>
+     * @see applicationData
      */
     static void setApplicationData(const KAboutData &aboutData);
 
@@ -316,11 +388,19 @@ public:
      * that was set in @p aboutData.
      */
     static void registerPluginData(const KAboutData &aboutData);
+
     /**
      * Return the KAboutData for the given plugin identified by @p componentName.
      */
     static KAboutData *pluginData(const QString &componentName);
 
+    /**
+     * Creates a @c KAboutData from the given @p plugin metadata
+     *
+     * @since 5.18
+     */
+    static KAboutData fromPluginMetaData(const KPluginMetaData &plugin);
+
 public:
     /**
      * Constructor.
@@ -353,16 +433,22 @@ public:
      *        information. The text can contain newlines. This string
      *        should be translated.
      *
-     * @param homePageAddress The component's homepage string.
-     *        Start the address with "http://". "http://some.domain" is
-     *        is correct, "some.domain" is not.
-     * IMPORTANT: if you set a home page address, this will change the "organization domain"
-     * of the application, which is used for automatic D-Bus registration.
-     * @see setOrganizationDomain
+     * @param homePageAddress The URL to the component's homepage, including
+     *        URL scheme. "http://some.domain" is correct, "some.domain" is
+     *        not. Since KDE Frameworks 5.17, https and other valid URL schemes
+     *        are also valid. See also the note below.
      *
-     * @param bugsEmailAddress The bug report email address string.
+     * @param bugAddress The bug report address string, an email address or a URL.
      *        This defaults to the kde.org bug system.
      *
+     * @note The @arg homePageAddress argument is used to derive a default organization
+     * domain for the application (which is used to register on the session D-Bus,
+     * locate the appropriate desktop file, etc.), by taking the host name and dropping
+     * the first component, unless there are less than three (e.g. "www.kde.org" -> "kde.org").
+     * Use both setOrganizationDomain(const QByteArray&) and setDesktopFileName() if their default values
+     * do not have proper values.
+     *
+     * @see setOrganizationDomain(const QByteArray&), setDesktopFileName(const QString&)
      */
     // KF6: remove constructor that includes catalogName, and put default
     //      values back in for shortDescription and licenseType
@@ -374,7 +460,7 @@ public:
                const QString &copyrightStatement = QString(),
                const QString &otherText = QString(),
                const QString &homePageAddress = QString(),
-               const QString &bugsEmailAddress = QLatin1String("submit@bugs.kde.org")
+               const QString &bugAddress = QStringLiteral("submit@bugs.kde.org")
               );
 
     /**
@@ -387,6 +473,11 @@ public:
      *        should be translated. Example: i18n("KWrite")
      *
      * @param version The component version string.
+     *
+     * Sets the property desktopFileName to "org.kde."+componentName and
+     * the property organizationDomain to "kde.org".
+     *
+     * @see setOrganizationDomain(const QByteArray&), setDesktopFileName(const QString&)
      */
     KAboutData(const QString &componentName,
                const QString &displayName,
@@ -464,25 +555,24 @@ public:
                           const QString &task = QString(),
                           const QString &emailAddress = QString(),
                           const QString &webAddress = QString(),
-                          const QString &ocsUserName = QString());
+                          const QString &ocsUsername = QString());
 
     /**
      * @brief Sets the name(s) of the translator(s) of the GUI.
      *
-     * Since this depends on the language, just use a dummy text marked for
-     * translation.
-     *
-     * The canonical use is:
+     * The canonical use with the ki18n framework is:
      *
      * \code
-     * setTranslator(ki18nc("NAME OF TRANSLATORS", "Your names"),
-     *               ki18nc("EMAIL OF TRANSLATORS", "Your emails"));
+     * setTranslator(i18nc("NAME OF TRANSLATORS", "Your names"),
+     *               i18nc("EMAIL OF TRANSLATORS", "Your emails"));
      * \endcode
      *
-     * The translator can then translate this dummy text with his name
-     * or with a list of names separated with ",".
-     * If there is no translation or the application is used with the
-     * default language, this function call is ignored.
+     * If you are using a KMainWindow this is done for you automatically.
+     *
+     * The name and emailAddress are treated as lists separated with ",".
+     *
+     * If the strings are empty or "Your names"/"Your emails"
+     * respectively they will be ignored.
      *
      * @param name the name(s) of the translator(s)
      * @param emailAddress the email address(es) of the translator(s)
@@ -662,27 +752,40 @@ public:
     /**
      * Defines the address where bug reports should be sent.
      *
-     * @param bugAddress The bug report email address string.
+     * @param bugAddress The bug report email address or URL.
      *        This defaults to the kde.org bug system.
      */
     KAboutData &setBugAddress(const QByteArray &bugAddress);
 
     /**
-     * Defines the Internet domain of the organization that wrote this application.
+     * Defines the domain of the organization that wrote this application.
      * The domain is set to kde.org by default, or the domain of the homePageAddress constructor argument,
      * if set.
      *
-     * Make sure to call setOrganizationDomain if your product is developed out of the
-     * kde.org version-control system.
+     * Make sure to call setOrganizationDomain(const QByteArray&) if your product
+     * is not developed inside the KDE community.
      *
-     * Used by the automatic registration to D-Bus done by KApplication and KUniqueApplication.
+     * Used e.g. for the registration to D-Bus done by KDBusService
+     * from the KDE Frameworks KDBusAddons module.
      *
-     * IMPORTANT: if the organization domain is set, the .desktop file that describes your
-     * application should have an entry like X-DBUS-ServiceName=reversed_domain.kmyapp
-     * For instance kwrite passes "http://www.kate-editor.org" as the homePageAddress so it needs
-     * X-DBUS-ServiceName=org.kate-editor.kwrite in its kwrite.desktop file.
+     * Calling this method has no effect on the value of the desktopFileName property.
      *
-     * @param domain the domain name, for instance kde.org, koffice.org, kdevelop.org, etc.
+     * @note: If your program should work as a D-Bus activatable service, the base name
+     * of the D-Bus service description file or of the desktop file you install must match
+     * the D-Bus "well-known name" for which the program will register.
+     * For example, KDBusService will use a name created from the reversed organization domain
+     * with the component name attached, so for an organization domain "bar.org" and a
+     * component name "foo" the name of an installed D-Bus service file needs to be
+     * "org.bar.foo.service" or the the name of the installed desktop file "org.bar.foo.desktop"
+     * (and the desktopFileName property accordingly set to "org.bar.foo").
+     * For still supporting the deprecated start of services via KToolInvocation,
+     * the desktop file needs to have an entry with the key "X-DBUS-ServiceName"
+     * and a value which matches the used D-Bus "well-known name" as just described,
+     * so with the above used values it needs a line "X-DBUS-ServiceName=org.bar.foo"
+     *
+     * @param domain the domain name, for instance kde.org, koffice.org, etc.
+     *
+     * @see setDesktopFileName(const QString&)
      */
     KAboutData &setOrganizationDomain(const QByteArray &domain);
 
@@ -720,7 +823,7 @@ public:
     /**
      * Returns the domain name of the organization that wrote this application.
      *
-     * Used by the automatic registration to D-Bus done by KApplication and KUniqueApplication.
+     * @see setOrganizationDomain(const QByteArray&)
      */
     QString organizationDomain() const;
 
@@ -784,8 +887,8 @@ public:
     QString homepage() const;
 
     /**
-     * Returns the email address for bugs.
-     * @return the email address where to report bugs.
+     * Returns the email address or URL for bugs.
+     * @return the address where to report bugs.
      */
     QString bugAddress() const;
 
@@ -892,7 +995,9 @@ public:
      *
      * Additionally, it will set the description to the command line parser, will add the help
      * option and if the QApplication has a version set (e.g. via KAboutData::setApplicationData)
-     * it will also add the version option
+     * it will also add the version option.
+     *
+     * Since 5.16 it also adds an option to set the desktop file name.
      *
      * @returns true if adding the options was successful; otherwise returns false.
      *
@@ -908,6 +1013,43 @@ public:
      */
     void processCommandLine(QCommandLineParser *parser);
 
+    /**
+     * Sets the base name of the desktop entry for this application.
+     *
+     * This is the file name, without the full path and without extension,
+     * of the desktop entry that represents this application according to
+     * the freedesktop desktop entry specification (e.g. "org.kde.foo").
+     *
+     * A default desktop file name is constructed when the KAboutData
+     * object is created, using the reverse domain name of the
+     * organizationDomain() and the componentName() as they are at the time
+     * of the KAboutData object creation.
+     * Call this method to override that default name. Typically this is
+     * done when also setOrganizationDomain(const QByteArray&) or setComponentName(const QString&)
+     * need to be called to override the initial values.
+     *
+     * The desktop file name can also be passed to the application at runtime through
+     * the @c desktopfile command line option which is added by setupCommandLine(QCommandLineParser*).
+     * This is useful if an application supports multiple desktop files with different runtime
+     * settings.
+     *
+     * @param desktopFileName The desktop file name of this application
+     *
+     * @sa desktopFileName()
+     * @sa organizationDomain()
+     * @sa componentName()
+     * @sa setupCommandLine(QCommandLineParser*)
+     * @since 5.16
+     **/
+    KAboutData &setDesktopFileName(const QString &desktopFileName);
+
+    /**
+     * @returns The desktop file name of this application (e.g. "org.kde.foo")
+     * @sa setDesktopFileName(const QString&)
+     * @since 5.16
+     **/
+    QString desktopFileName() const;
+
 private:
 
     friend void KCrash::defaultCrashHandler(int sig);
